@@ -815,11 +815,29 @@ bool CApplication::CreateVideoSurface()
     m_private->glcontext = SDL_GL_CreateContext(m_private->window);
 
     int vsync = 0;
-    if (GetConfigFile().GetIntProperty("Experimental", "VSync", vsync))
+    if (GetConfigFile().GetIntProperty("Setup", "VSync", vsync))
     {
-        SDL_GL_SetSwapInterval(vsync);
+        while (SDL_GL_SetSwapInterval(vsync) == -1)
+        {
+            switch(vsync)
+            {
+                case -1: //failed with adaptive sync?
+                    GetLogger()->Warn("Adaptive sync not supported.\n");
+                    vsync = 1;
+                    break;
+                case 1: //failed with VSync enabled?
+                    GetLogger()->Warn("Couldn't enable VSync.\n");
+                    vsync = 0;
+                    break;
+                case 0: //failed with VSync disabled?
+                    GetLogger()->Warn("Couldn't disable VSync.\n");
+                    vsync = 1;
+                    break;
+            }
+        }
+        GetConfigFile().SetIntProperty("Setup", "VSync", vsync);
 
-        GetLogger()->Info("Using Vsync: %s\n", (vsync ? "true" : "false"));
+        GetLogger()->Info("Using Vsync: %s\n", (vsync == -1 ? "adaptive" : (vsync ? "true" : "false")));
     }
 
     return true;
@@ -833,6 +851,27 @@ bool CApplication::ChangeVideoConfig(const Gfx::DeviceConfig &newConfig)
     SDL_SetWindowSize(m_private->window, m_deviceConfig.size.x, m_deviceConfig.size.y);
     SDL_SetWindowFullscreen(m_private->window, m_deviceConfig.fullScreen ? SDL_WINDOW_FULLSCREEN : 0);
 
+    int vsync = m_engine->GetVSync();
+    while (SDL_GL_SetSwapInterval(vsync) == -1)
+    {
+        switch(vsync)
+        {
+            case -1: //failed with adaptive sync?
+                GetLogger()->Warn("Adaptive sync not supported.\n");
+                vsync = 1;
+                break;
+            case 1: //failed with VSync enabled?
+                GetLogger()->Warn("Couldn't enable VSync.\n");
+                vsync = 0;
+                break;
+            case 0: //failed with VSync disabled?
+                GetLogger()->Warn("Couldn't disable VSync.\n");
+                vsync = 1;
+                break;
+        }
+    }
+    m_engine->SetVSync(vsync);
+
     m_device->ConfigChanged(m_deviceConfig);
 
     m_eventQueue->AddEvent(Event(EVENT_RESOLUTION_CHANGED));
@@ -1718,6 +1757,11 @@ char CApplication::GetLanguageChar() const
         case LANGUAGE_RUSSIAN:
             langChar = 'R';
             break;
+
+        case LANGUAGE_PORTUGUESE_BRAZILIAN:
+            langChar = 'B';
+            break;
+
     }
     return langChar;
 }
@@ -1774,6 +1818,10 @@ void CApplication::SetLanguage(Language language)
             {
                 m_language = LANGUAGE_RUSSIAN;
             }
+            else if (strncmp(envLang,"pt",2) == 0)
+            {
+                m_language = LANGUAGE_PORTUGUESE_BRAZILIAN;
+            }
             else
             {
                 GetLogger()->Warn("Enviromnent locale ('%s') is not supported, setting default language\n", envLang);
@@ -1812,6 +1860,10 @@ void CApplication::SetLanguage(Language language)
         case LANGUAGE_RUSSIAN:
             locale = "ru_RU.utf8";
             break;
+
+        case LANGUAGE_PORTUGUESE_BRAZILIAN:
+            locale = "pt_BR.utf8";
+            break;
     }
 
     std::string langStr = "LANGUAGE=";
@@ -1832,7 +1884,12 @@ void CApplication::SetLanguage(Language language)
     // Update C++ locale
     try
     {
+#if defined(_MSC_VER) && defined(_DEBUG)
+        // Avoids failed assertion in VS debugger
+        throw -1;
+#else
         std::locale::global(std::locale(systemLocale.c_str()));
+#endif
     }
     catch (...)
     {
