@@ -1072,6 +1072,13 @@ public:
     bool            GetTerrainShadows();
     //@}
 
+    //@{
+    //! Management of vertical synchronization
+    // NOTE: This is an user configuration setting
+    void            SetVSync(int value);
+    int             GetVSync();
+    //@}
+
     //@{
     //! Management of shadow color
     // NOTE: This is a setting configurable only in INI file
@@ -1336,6 +1343,9 @@ protected:
     //! Texture bias for sampling shadow maps
     Math::Matrix    m_shadowBias;
 
+    //! Vertical synchronization controll
+    int m_vsync;
+
     //! World matrix for 2D interface
     Math::Matrix    m_matWorldInterface;
     //! Projection matrix for 2D interface
